@@ -1,10 +1,11 @@
 """
-This module defines colors to be used for graphs.
+This module defines colors to be used for graphs. Look at the Color class.
 """
 
-from typing import Any, Iterable, Tuple, Union
+from typing import Any, Iterable, Iterator, Tuple, Union
+from Viper.frozendict import frozendict
 
-__all__ = ["chart", "Color"]
+__all__ = ["Color"]
 
 
 
@@ -152,7 +153,7 @@ __chart_init = {
     'white' : (1.0, 1.0, 1.0)
 }
 
-chart : dict[str, "Color"] = {}
+chart : frozendict[str, "Color"]
 
 
 
@@ -171,11 +172,26 @@ class ColorType(type):
         try:
             return super().__getattribute__(name)
         except AttributeError:
-            if name in chart:
-                return chart[name]
+            if name in self.chart:
+                return self.chart[name]
             else:
                 raise AttributeError("type object " + repr(super().__getattribute__("__name__")) + " has no attribute " + repr(name))
-
+            
+    def __dir__(self) -> list[str]:
+        """
+        Implements dir(Color).
+        """
+        return list(super().__dir__()) + list(self.chart)
+    
+    @property
+    def names(self) -> list[str]:
+        """
+        Returns the list of available color names.
+        """
+        return list(self.chart)
+    
+    chart : frozendict[str, "Color"] = frozendict()
+    inverted_chart : frozendict[tuple[int, int, int], str] = frozendict()
 
 
 
@@ -184,6 +200,11 @@ class Color(metaclass = ColorType):
 
     """
     This class is used to describe colors.
+    They can be defined using RGB values (three float (0:1) or three int [0:255]) or using the color name:
+    >>> Color(1.0, 1.0, 1.0) is Color.white
+    True
+    
+    Use Color.names to list all the available color names.
     """
 
     __slots__ = ("R", "G", "B")
@@ -194,9 +215,9 @@ class Color(metaclass = ColorType):
         if isinstance(r, int) and isinstance(g, int) and isinstance(b, int):
             r, g, b = r / 255, g / 255, b / 255
         r, g, b = float(r), float(g), float(b)
-        for name, value in chart.items():
-            if isinstance(value, Color) and (value.R, value.G, value.B) == (r, g, b):
-                return value
+        R, G, B = round(r * 255), round(g * 255), round(b * 255)
+        if (R, G, B) in Color.inverted_chart:
+            return Color.chart[Color.inverted_chart[R, G, B]]
         return super().__new__(cls)
 
 
@@ -213,10 +234,9 @@ class Color(metaclass = ColorType):
         """
         Implements str(self).
         """
-        for name, value in chart.items():
-            if Color(value.R, value.G, value.B) == self:
-                return "color." + name
-        return "color" + str((self.R, self.G, self.B))
+        if self.to_int() in Color.inverted_chart:
+            return "Color." + Color.inverted_chart[self.to_int()]
+        return "Color" + str((self.R, self.G, self.B))
     
 
     __repr__ = __str__
@@ -300,6 +320,30 @@ class Color(metaclass = ColorType):
             return Color(min(1.0, max(0.0, self.R ** o)), min(1.0, max(0.0, self.G ** o)), min(1.0, max(0.0, self.B ** o)))
         else:
             return NotImplemented
+        
+
+    def negative_to(self, ref : "Color") -> "Color":
+        """
+        Returns the negative of this color relative to another color. (Equivalent to -self relatively to any whitescale color)
+        """
+        if not isinstance(ref, Color):
+            raise TypeError(f"Exepcted Color, got '{type(ref).__name__}'")
+        M = Color(0.5, 0.5, 0.5)
+        if ref == M:
+            raise ZeroDivisionError(f"Cannot compute negative relative to central Color ({M})")
+        SM : tuple[float, float, float] = tuple(Mi - Si for Mi, Si in zip(M, self))
+        RM : tuple[float, float, float] = tuple(Mi - Ri for Mi, Ri in zip(M, ref))
+        fact = 2 * sum(a * b for a, b in zip(SM, RM)) / sum(b ** 2 for b in RM)
+        return Color(*(max(min(Si + fact * RMi, 1.0), 0.0) for Si, RMi in zip(self, RM)))
+
+
+    def __iter__(self) -> Iterator[float]:
+        """
+        Implements iter(self).
+        """
+        yield self.R
+        yield self.G
+        yield self.B
 
     
     def to_int(self) -> Tuple[int, int, int]:
@@ -338,6 +382,7 @@ class Color(metaclass = ColorType):
 
 
 # Convert color vectors of the __chart_init to Color objects in the real chart
-chart.update((name, Color(*value)) for name, value in __chart_init.items())
+ColorType.chart = frozendict((name, Color(*value)) for name, value in __chart_init.items())
+ColorType.inverted_chart = frozendict((color.to_int(), name) for name, color in ColorType.chart.items())
 
-del Any, Tuple, Union, ColorType, Iterable, __chart_init
\ No newline at end of file
+del Any, Tuple, Union, ColorType, Iterable, Iterator, __chart_init, frozendict
\ No newline at end of file
