@@ -2,7 +2,7 @@
 This package defines the graph-related subclasses that modelize the flow of data.
 """
 
-from ....logger import logger
+from .....logger import logger
 
 
 
