@@ -2,7 +2,7 @@
 This module contains integration protocols for this behavioral package.
 """
 
-from ....logger import logger
+from .....logger import logger
 from ...utils import chrono
 from ..filesystem.integration import NewFile
 from . import entities, relations
