@@ -2,7 +2,7 @@
 This module contains Edge and Arrow subclasses for this behavioral package.
 """
 
-from ....logger import logger
+from .....logger import logger
 from ...graph import Edge
 from ..execution.entities import Process
 from ..filesystem.entities import File
