@@ -1,26 +1,37 @@
 """
-This module defines MetaGraphs, MetaVertices, MetaEdges and MetaArrows.
+This module defines MetaGraphs, MetaVertices, MetaEdges and MetaArrows. They allow you to create graph patterns.
+Look at these classes.
 """
 
-from typing import Any, Callable, Iterable, Iterator, Optional, Self
-from ...bakery.source.graph import Arrow, Edge, Graph, UniqueVertex, Vertex
+from types import UnionType
+from typing import Any, Iterable, Iterator, Optional
+
+from baguette.bakery.source.graph import Edge
+from baguette.croutons.source.evaluator import Evaluator
+
 from ...bakery.source.colors import Color
+from ...bakery.source.graph import Arrow, Edge, Graph, UniqueVertex, Vertex
 from ...bakery.source.types.execution import *
-from ...bakery.source.types.network import *
 from ...bakery.source.types.filesystem import *
-
-
-
+from ...bakery.source.types.network import *
+from .evaluator import Evaluator
 
 
 class MetaVertex(UniqueVertex):
 
     """
-    This class describes a type vertex. One or multiple Vertex subclasses can be associated to it.
+    This class describes a type vertex. One or multiple Vertex subclasses can be associated to it as well as additional conditions.
+    This class defines properties cls and condition which can be efficiently affected with a special syntax:
+    >>> MV = MetaVertex()
+    >>> MV.cls = (File, )
+    >>> MV.condition = Evaluator("lambda x : x.name.endswith('.exe')")
+
+    is equivalent to:
+    >>> MV = MetaVertex[File]("lambda x : x.name.endswith('.exe')")
     """
 
     @classmethod
-    def __class_getitem__(cls, cls_init):
+    def __class_getitem__(cls, cls_init : type[Vertex] | tuple[type[Vertex], ...] | UnionType):
         try:
             Mv = cls()
             Mv.cls = cls_init
@@ -36,15 +47,17 @@ class MetaVertex(UniqueVertex):
 
     __pickle_slots__ = {
         "cls",
-        "condition"
+        "condition",
+        "color"
     }
 
-    def __init__(self, *, c: Color | None = None, parent: Optional[Vertex] = None) -> None:
+    def __init__(self, *, parent: Optional[Vertex] = None) -> None:
         from ...bakery.source.colors import Color
-        super().__init__(c=c, parent=parent)
-        self.__class : type[Vertex] = Vertex
-        self.__condition : Callable[[Vertex], bool] | bool = True
-        self.__color : Color | None = c
+        from .evaluator import Evaluator
+        super().__init__(parent=parent)
+        self.__class : tuple[type[Vertex], ...] = (Vertex, )
+        self.__condition : Evaluator[Vertex, bool] | None = None
+        self.__color : Color | None = None
         self.edges : set[MetaEdge]
 
     @property
@@ -53,15 +66,9 @@ class MetaVertex(UniqueVertex):
         The color of this MetaVertex. Defaults to the average of the colors of all its Vertex classes.
         """
         from ...bakery.source.colors import Color
-        from ...bakery.source.config import ColorMap
-        if self.__color !=  None:
+        if self.__color is not None:
             return self.__color
-        if isinstance(self.cls, type):
-            if not hasattr(self.cls, "color") or not isinstance(self.cls.color, Color):
-                return ColorMap.get_config().get_color(self.cls)
-            return self.cls.color
-        else:
-            return Color.average(*[(cls.color if hasattr(cls, "color") and isinstance(cls.color, Color) else ColorMap.get_config().get_color(cls)) for cls in self.cls.__args__])
+        return Color.average(*[(cls.default_color) for cls in self.cls])
     
     @color.setter
     def color(self, c : Color):
@@ -69,79 +76,114 @@ class MetaVertex(UniqueVertex):
         if not isinstance(c, Color):
             raise TypeError("Expected Color, got " + repr(type(c).__name__))
         self.__color = c
+
+    @color.deleter
+    def color(self):
+        self.__color = None
     
     @property
-    def cls(self) -> type[Vertex]:
+    def cls(self) -> tuple[type[Vertex], ...]:
         """
-        The class that his vertex represents.
+        The classes that this vertex represents.
         """
         return self.__class
     
     @cls.setter
-    def cls(self, cls : type[Vertex]):
+    def cls(self, cls : type[Vertex] | tuple[type[Vertex], ...] | UnionType):
         """
-        Sets the class associated with this vertex.
+        Sets the class(es) associated with this vertex.
         """
-        from typing import _UnionGenericAlias
         from types import UnionType
-        from .utils import class_union
+
         from ...bakery.source.graph import Vertex
-        if isinstance(cls, type) and issubclass(cls, Vertex):
-            self.__class = cls
-        elif isinstance(cls, _UnionGenericAlias):
-            cls = class_union(*cls.__args__)
-            for c in cls.__args__:
+        if isinstance(cls, UnionType):
+            args : tuple[type[Vertex]] = cls.__args__
+            for c in args:
                 if not isinstance(c, type) or not issubclass(c, Vertex):
-                    raise TypeError("Expected subclasses of Vertex , got " + repr(c))
-            self.__class = cls
-        elif isinstance(cls, UnionType):
-            for c in cls.__args__:
+                    raise TypeError("Expected subclass of Vertex or tuple of subclasses, got a " + repr(c))
+            self.__class = args
+        elif isinstance(cls, type) and issubclass(cls, Vertex):
+            self.__class = (cls, )
+        elif isinstance(cls, tuple):
+            for c in cls:
                 if not isinstance(c, type) or not issubclass(c, Vertex):
-                    raise TypeError("Expected subclasses of Vertex , got " + repr(c))
+                    raise TypeError("Expected subclass of Vertex or tuple of subclasses, got a " + repr(c))
             self.__class = cls
         else:
-            raise TypeError("Expected subclass of Vertex , got " + repr(cls))
+            raise TypeError("Expected subclass of Vertex or tuple of subclasses, got " + repr(cls))
+    
+    @cls.deleter
+    def cls(self):
+        from ...bakery.source.graph import Vertex
+        self.__class = (Vertex, )
 
     @property
-    def condition(self) -> Callable[[Vertex], bool] | bool:
+    def condition(self) -> Evaluator[Vertex, bool] | None:
         """
-        An additional condition to check when trying to match a Vertex to this MetaVertex.
+        An additional and optional condition to check when trying to match a Vertex to this MetaVertex.
         """
         return self.__condition
     
     @condition.setter
-    def condition(self, cond : Callable[[Vertex], bool] | bool):
+    def condition(self, cond : Evaluator[Vertex, bool] | str):
         """
         Sets the additional condition function for this MetaVertex.
         """
-        if not callable(cond) and not isinstance(cond, bool):
-            raise TypeError("Expected callable or bool, got " + repr(type(cond).__name__))
+        from .evaluator import Evaluator
+        if isinstance(cond, str):
+            try:
+                cond = Evaluator(cond)
+            except SyntaxError as e:
+                raise e from None
+        if not isinstance(cond, Evaluator):
+            raise TypeError(f"Expected Evaluator or str, got '{type(cond).__name__}'")
         self.__condition = cond
+
+    @condition.deleter
+    def condition(self):
+        self.__condition = None
     
     def match(self, v : Vertex) -> bool:
         """
         Returns True if the Vertex v has a matching type.
         """
-        return isinstance(v, self.__class) and (self.__condition(v) if callable(self.__condition) else self.__condition)
+        return isinstance(v, self.__class) and (self.__condition(v) if self.__condition else True)
     
     def __get_cls_str(self) -> str:
         """
         Returns a string to display the class of a MetaVertex.
         """
-        if isinstance(self.__class, type):
-            return self.__class.__name__
-        else:
-            return str(self.__class)
+        return " | ".join(c.__name__ for c in self.__class)
+    
+    def __call__(self, cond : Evaluator[Vertex, bool] | str) :
+        """
+        Implements self(cond). Sets the condition and returns self.
+        """
+        try:
+            self.condition = cond
+            return self
+        except BaseException as e:
+            raise e from None
+        
+    def __getstate__(self):
+        s = super().__getstate__()
+        if not s["condition"]:
+            s.pop("condition")
+        return s
+    
+    def __setstate__(self, state : dict[str, Any]):
+        self.__condition = None
+        super().__setstate__(state)
 
     @property
     def label(self) -> str:
-        return "Vertex[" + self.__get_cls_str() + "]" + ("{" + str(self.__condition) + "}" if callable(self.__condition) else "")
+        return f"Vertex[{self.__get_cls_str()}]" + (f"({str(self.__condition)})" if self.__condition else "")
 
     def __str__(self) -> str:
-        return type(self).__name__ + "[" + self.__get_cls_str() + "]" + ("*" if callable(self.__condition) else "")
+        return f"{type(self).__name__}[{self.__get_cls_str()}]" + ("*" if self.__condition else "")
     
     def __repr__(self) -> str:
-        return type(self).__name__ + "[" + self.__get_cls_str() + "]" + ("{" + str(self.__condition) + "}" if callable(self.__condition) else "")
+        return f"{type(self).__name__}[{self.__get_cls_str()}]" + (f"({repr(self.__condition.code)})" if self.__condition else "")
     
 
 
@@ -152,27 +194,40 @@ class MetaEdge(Edge):
     """
     This class describes a type edge. One or multiple Edge subclasses can be associated to it.
     Note that a MetaEdge can represent a subclass of Arrow and in this case, the match can be made in both directions.
+    Like MetaVertices, MetaEdges have special syntaxes for setting the cls and condition attributes:
+    >>> U = MetaVertex[Directory]
+    >>> V = MetaVertex[File]
+    >>> UV = MetaEdge(U, V)
+    >>> UV.cls = (Contains, )
+    >>> UV.condition = Evaluator("lambda x : True")     # Dummy condition
+
+    is equivalent to:
+    >>> U = MetaVertex[Directory]
+    >>> V = MetaVertex[File]
+    >>> UV = MetaEdge(U, V)[contains]("lambda x : True")
     """
 
     __slots__ = {
         "__class" : "The class that this MetaEdge represents.",
         "__condition" : "An additional condition function. Takes an Edge as an input and tells whether or not it can be a valid match (does not need to check type)",
-        "__color" : "The Color forcefully set to this MetaVertex."
+        "__color" : "The Color forcefully set to this MetaEdge."
     }
 
     __pickle_slots__ = {
         "cls",
-        "condition"
+        "condition",
+        "color"
     }
 
-    def __init__(self, source: MetaVertex, destination: MetaVertex, *, c: Color | None = None, auto_write: bool = True) -> None:
+    def __init__(self, source: MetaVertex, destination: MetaVertex, *, auto_write: bool = True) -> None:
         from ...bakery.source.colors import Color
+        from .evaluator import Evaluator
         if not isinstance(source, MetaVertex) or not isinstance(destination, MetaVertex):
             raise TypeError("Expected two MetaVertices, got " + repr(type(source).__name__) + " and " + repr(type(destination).__name__))
-        super().__init__(source, destination, c=c, auto_write=auto_write)
-        self.__class : type[Edge] = Edge
-        self.__condition : Callable[[Edge], bool] | bool = True
-        self.__color : Color | None = c
+        super().__init__(source, destination, auto_write=auto_write)
+        self.__class : tuple[type[Edge], ...] = (Edge, )
+        self.__condition : Evaluator[Edge, bool] | None = None
+        self.__color : Color | None = None
         self.source : MetaVertex
         self.destination : MetaVertex
 
@@ -182,15 +237,13 @@ class MetaEdge(Edge):
         The color of this MetaEdge. Defaults to the average of the colors of all its Edge classes.
         """
         from ...bakery.source.colors import Color
-        from ...bakery.source.config import ColorMap
-        if self.__color !=  None:
+        if self.__color != None:
             return self.__color
-        if isinstance(self.cls, type):
-            if not hasattr(self.cls, "color") or not isinstance(self.cls.color, Color):
-                return ColorMap.get_config().get_color(self.cls)
-            return self.cls.color
+        edge_colors = [cls.default_color for cls in self.cls if not cls.blend_vertices_colors]
+        if edge_colors:
+            return Color.average(*edge_colors)
         else:
-            return Color.average(*[(cls.color if hasattr(cls, "color") and isinstance(cls.color, Color) else ColorMap.get_config().get_color(cls)) for cls in self.cls.__args__])
+            return Color.average(self.source.color, self.destination.color)
     
     @color.setter
     def color(self, c : Color):
@@ -198,84 +251,119 @@ class MetaEdge(Edge):
         if not isinstance(c, Color):
             raise TypeError("Expected Color, got " + repr(type(c).__name__))
         self.__color = c
+
+    @color.deleter
+    def color(self):
+        self.__color = None
     
     @property
-    def cls(self) -> type[Edge]:
+    def cls(self) -> tuple[type[Edge], ...]:
         """
-        The class that his edge represents.
+        The classes that this edge represents.
         """
         return self.__class
     
     @cls.setter
-    def cls(self, cls : type[Edge]):
+    def cls(self, cls : type[Edge] | tuple[type[Edge], ...] | UnionType):
         """
-        Sets the class associated with this edge.
+        Sets the class(es) associated with this edge.
         """
-        from typing import _UnionGenericAlias
         from types import UnionType
-        from .utils import class_union
+
         from ...bakery.source.graph import Edge
-        if isinstance(cls, type) and issubclass(cls, Edge):
-            self.__class = cls
-        elif isinstance(cls, _UnionGenericAlias):
-            cls = class_union(*cls.__args__)
-            for c in cls.__args__:
+        if isinstance(cls, UnionType):
+            args : tuple[type[Edge]] = cls.__args__
+            for c in args:
                 if not isinstance(c, type) or not issubclass(c, Edge):
-                    raise TypeError("Expected subclasses of Edge , got " + repr(c))
-            self.__class = cls
-        elif isinstance(cls, UnionType):
-            for c in cls.__args__:
+                    raise TypeError("Expected subclass of Edge or tuple of subclasses, got a " + repr(c))
+            self.__class = args
+        elif isinstance(cls, type) and issubclass(cls, Edge):
+            self.__class = (cls, )
+        elif isinstance(cls, tuple):
+            for c in cls:
                 if not isinstance(c, type) or not issubclass(c, Edge):
-                    raise TypeError("Expected subclasses of Edge , got " + repr(c))
+                    raise TypeError("Expected subclasses of Edge or tuple of subclasses, got a " + repr(c))
             self.__class = cls
+        else:
+            raise TypeError("Expected subclass of Edge or tuple of subclasses, got " + repr(cls))
+        
+    @cls.deleter
+    def cls(self):
+        from ...bakery.source.graph import Edge
+        self.__class = (Edge, )
         
     @property
-    def condition(self) -> Callable[[Edge], bool] | bool:
+    def condition(self) -> Evaluator[Edge, bool] | None:
         """
         An additional condition to check when trying to match an Edge to this MetaEdge.
         """
         return self.__condition
     
     @condition.setter
-    def condition(self, cond : Callable[[Edge], bool] | bool):
+    def condition(self, cond : Evaluator[Edge, bool] | str):
         """
         Sets the additional condition function for this MetaEdge.
         """
-        if not callable(cond) and not isinstance(cond, bool):
-            raise TypeError("Expected callable or bool, got " + repr(type(cond).__name__))
+        from .evaluator import Evaluator
+        if isinstance(cond, str):
+            try:
+                cond = Evaluator(cond)
+            except SyntaxError as e:
+                raise e from None
+        if not isinstance(cond, Evaluator):
+            raise TypeError(f"Expected Evaluator or str, got '{type(cond).__name__}'")
         self.__condition = cond
     
+    @condition.deleter
+    def condition(self):
+        self.__condition = None
+    
     def match(self, e : Edge) -> bool:
         """
         Returns True if the Egde e has a matching type.
         """
-        return isinstance(e, self.__class) and (self.__condition(e) if callable(self.__condition) else self.__condition)
+        return isinstance(e, self.__class) and (self.__condition(e) if self.__condition else True)
     
     def __get_cls_str(self) -> str:
         """
         Returns a string to display the class of a MetaEdge.
         """
-        if isinstance(self.__class, type):
-            return self.__class.__name__
-        else:
-            return str(self.__class)
+        return " | ".join(c.__name__ for c in self.cls)
+    
+    def __getstate__(self):
+        s = super().__getstate__()
+        if not s["condition"]:
+            s.pop("condition")
+        return s
+    
+    def __setstate__(self, state : dict[str, Any]):
+        self.__condition = None
+        super().__setstate__(state)
 
     @property
     def label(self) -> str:
-        return "Edge[" + self.__get_cls_str() + "]" + ("{" + str(self.__condition) + "}" if callable(self.__condition) else "")
+        return f"Edge[{self.__get_cls_str()}]" + (f"({repr(self.__condition.code)})" if self.__condition else "")
 
     def __str__(self) -> str:
-        return type(self).__name__ + "[" + self.__get_cls_str() + "]" + ("*" if callable(self.__condition) else "")
+        return f"{type(self).__name__}[{self.__get_cls_str()}]" + ("*" if self.__condition else "")
     
     def __repr__(self) -> str:
-        return type(self).__name__ + "[" + self.__get_cls_str() + "]" + ("{" + str(self.__condition) + "}" if callable(self.__condition) else "")
+        return f"{type(self).__name__}[{self.__get_cls_str()}]" + (f"({repr(self.__condition.code)})" if self.__condition else "")
 
-    def __getitem__(self, cls : type[Edge]) -> Self:
+    def __getitem__(self, cls : type[Edge] | tuple[type[Edge], ...] | UnionType):
         try:
-            self.__class = cls
+            self.cls = cls
             return self
         except BaseException as E:
             raise E from None
+        
+    def __call__(self, cond : Evaluator[Edge, bool] | str):
+        try:
+            self.condition = cond
+            return self
+        except BaseException as e:
+            raise e from None
+
 
 
 
@@ -284,6 +372,7 @@ class MetaArrow(Arrow, MetaEdge):
 
     """
     This class describes a type arrow. One or multiple Arrow subclasses can be associated to it.
+    It has the same syntax rules as MetaEdges.
     """
 
     __slots__ = {
@@ -296,88 +385,110 @@ class MetaArrow(Arrow, MetaEdge):
         "condition"
     }
 
-    def __init__(self, source: MetaVertex, destination: MetaVertex, *, c: Color | None = None, auto_write: bool = True) -> None:
-        super().__init__(source, destination, c=c, auto_write=auto_write)
-        self.__class : type[Arrow] = Arrow
-        self.__condition : Callable[[Arrow], bool] | bool = True
-    
+    def __init__(self, source: MetaVertex, destination: MetaVertex, *, auto_write: bool = True) -> None:
+        from ...bakery.source.graph import Arrow
+        from .evaluator import Evaluator
+        super().__init__(source, destination, auto_write=auto_write)
+        self.__class : tuple[type[Arrow], ...] = (Arrow, )
+        self.__condition : Evaluator[Arrow, bool] | None = None
+
     @property
-    def cls(self) -> type[Arrow]:
+    def cls(self) -> tuple[type[Arrow], ...]:
         """
-        The class that his arrow represents.
+        The classes that this arrow represents.
         """
         return self.__class
     
     @cls.setter
-    def cls(self, cls : type[Arrow]):
+    def cls(self, cls : type[Arrow] | tuple[type[Arrow], ...] | UnionType):
         """
-        Sets the class associated with this arrow.
+        Sets the class(es) associated with this arrow.
         """
-        from typing import _UnionGenericAlias
         from types import UnionType
-        from .utils import class_union
-        from ...bakery.source.graph import Arrow
-        if isinstance(cls, type) and issubclass(cls, Arrow):
-            self.__class = cls
-        elif isinstance(cls, _UnionGenericAlias):
-            cls = class_union(*cls.__args__)
-            for c in cls.__args__:
-                if not isinstance(c, type) or not issubclass(c, Arrow):
-                    raise TypeError("Expected subclasses of Arrow , got " + repr(c))
-            self.__class = cls
-        elif isinstance(cls, UnionType):
-            for c in cls.__args__:
-                if not isinstance(c, type) or not issubclass(c, Arrow):
-                    raise TypeError("Expected subclasses of Arrow , got " + repr(c))
-            self.__class = cls
 
+        from ...bakery.source.graph import Arrow
+        if isinstance(cls, UnionType):
+            args : tuple[type[Arrow]] = cls.__args__
+            for c in args:
+                if not isinstance(c, type) or not issubclass(c, Arrow):
+                    raise TypeError("Expected subclass of Arrow or tuple of subclasses, got a " + repr(c))
+            self.__class = args
+        elif isinstance(cls, type) and issubclass(cls, Arrow):
+            self.__class = (cls, )
+        elif isinstance(cls, tuple):
+            for c in cls:
+                if not isinstance(c, type) or not issubclass(c, Arrow):
+                    raise TypeError("Expected subclasses of Arrow or tuple of subclasses, got a " + repr(c))
+            self.__class = cls
+        else:
+            raise TypeError("Expected subclass of Arrow or tuple of subclasses, got " + repr(cls))
+        
+    @cls.deleter
+    def cls(self):
+        from ...bakery.source.graph import Arrow
+        self.__class = (Arrow, )
+        
     @property
-    def condition(self) -> Callable[[Arrow], bool] | bool:
+    def condition(self) -> Evaluator[Arrow, bool] | None:
         """
         An additional condition to check when trying to match an Arrow to this MetaArrow.
         """
         return self.__condition
     
     @condition.setter
-    def condition(self, cond : Callable[[Arrow], bool] | bool):
+    def condition(self, cond : Evaluator[Arrow, bool] | str):
         """
         Sets the additional condition function for this MetaArrow.
         """
-        if not callable(cond) and not isinstance(cond, bool):
-            raise TypeError("Expected callable or bool, got " + repr(type(cond).__name__))
+        from .evaluator import Evaluator
+        if isinstance(cond, str):
+            try:
+                cond = Evaluator(cond)
+            except SyntaxError as e:
+                raise e from None
+        if not isinstance(cond, Evaluator):
+            raise TypeError(f"Expected Evaluator or str, got '{type(cond).__name__}'")
         self.__condition = cond
     
-    def match(self, a : Edge) -> bool:
+    @condition.deleter
+    def condition(self):
+        self.__condition = None
+    
+    def __setstate__(self, state : dict[str, Any]):
+        self.__condition = None
+        super().__setstate__(state)
+    
+    def match(self, a : Arrow) -> bool:
         """
         Returns True if the Arrow a has a matching type.
         """
-        return isinstance(a, self.__class) and (self.__condition(a) if callable(self.__condition) else self.__condition)
+        return isinstance(a, self.__class) and (self.__condition(a) if self.__condition else True)
 
     def __get_cls_str(self) -> str:
         """
         Returns a string to display the class of a MetaArrow.
         """
-        if isinstance(self.__class, type):
-            return self.__class.__name__
-        else:
-            return str(self.__class)
+        return " | ".join(c.__name__ for c in self.cls)
 
     @property
     def label(self) -> str:
-        return "Arrow[" + self.__get_cls_str() + "]" + ("{" + str(self.__condition) + "}" if callable(self.__condition) else "")
+        return f"Arrow[{self.__get_cls_str()}]" + (f"({repr(self.__condition.code)})" if self.__condition else "")
     
     def __str__(self) -> str:
-        return type(self).__name__ + "[" + self.__get_cls_str() + "]" + ("*" if callable(self.__condition) else "")
+        return f"{type(self).__name__}[{self.__get_cls_str()}]" + ("*" if self.__condition else "")
     
     def __repr__(self) -> str:
-        return type(self).__name__ + "[" + self.__get_cls_str() + "]" + ("{" + str(self.__condition) + "}" if callable(self.__condition) else "")
+        return f"{type(self).__name__}[{self.__get_cls_str()}]" + (f"({repr(self.__condition.code)})" if self.__condition else "")
     
-    def __getitem__(self, cls : type[Arrow]) -> Self:
+    def __getitem__(self, cls : type[Arrow] | tuple[type[Arrow], ...] | UnionType):
+        return super().__getitem__(cls)
+    
+    def __call__(self, cond: Evaluator[Arrow, bool] | str):
         try:
-            self.__class = cls
+            self.condition = cond
             return self
-        except BaseException as E:
-            raise E from None
+        except BaseException as e:
+            raise e from None
     
 
 
@@ -387,6 +498,13 @@ class MetaGraph(Graph):
 
     """
     This particular type of graph can only contain MetaVertices, MetaEdges or MetaArrows. It can be used for normal graph exploration.
+    Note that contrary to normal Graphs, MetaGraphs can hold named vertices and edges:
+    >>> MG = MetaGraph()
+    >>> MG.File = MetaVertex[File]
+    >>> len(MG.vertices)
+    1
+    >>> mG.File
+    MetaVertex[File]
     """
 
     __slots__ = {
@@ -678,9 +796,10 @@ class MetaGraph(Graph):
         """
         Searches through g for all occurences of a subgraph that matches the metagraph.
         """
-        from ...bakery.source.graph import Graph
         from typing import Iterator
 
+        from ...bakery.source.graph import Graph
+
         # subgraphs : list[dict[MetaVertex | MetaEdge, Vertex | Edge]] = [{}]
         # next_subgraphs : list[dict[MetaVertex | MetaEdge, Vertex | Edge]] = []
 
