@@ -1,5 +1,5 @@
 """
-This module contains utilities to manage BAGUETTE data.
+This module contains utilities to manage BAGUETTE data. For more information, look at the BaguetteRack class.
 """
 
 from pathlib import Path
@@ -8,6 +8,7 @@ from typing import Any, Literal, TypeVar
 
 from .croutons.source.metagraph import MetaGraph
 from .bakery.source.filters import Filter
+from .bakery.source.colors import Color
 
 __all__ = ["BaguetteRack", "TimeoutExit"]
 
@@ -26,16 +27,12 @@ class BaguetteRack:
 
     """
     Baguette racks are just a utility to organize baguettes easily.
+    They have MANY properties that can be changed to manage BAGUETTEs.
     Remember to export() them before deleting them, as this will write the index file.
     """
 
     __slots__ = {
         "__working_directory" : "The absolute path to the working directory",
-        "__index" : "The path to the index file. Can be relative to __working_directory",
-        "__report" : "The path to the Cuckoo report file. Can be relative to __working_directory",
-        "__baguette" : "The path to the baguette file. Can be relative to __working_directory",
-        "__visual" : "The path to the Gephi file. Can be relative to __working_directory",
-        "__extracted" : "The path to the results of the metagraph search. Can be relative to __working_directory",
         "__perf" : "A boolean indicating if a performance report should be returned",
         "__filters" : "The list if filters applied during the compilation phase",
         "__patterns" : "The list of MetaGraphs to search for during the extraction phase",
@@ -45,7 +42,26 @@ class BaguetteRack:
         "__baked" : "Indicates if the baguette has been successfully baked",
         "__toasted" : "Indicates if the baguette has been successfully toasted",
         "__verbosity" : "The verbosity level to apply when baking or toasting", 
-        "__maxtime" : "The maximum amount of time to spend on this baguette"
+        "__maxtime" : "The maximum amount of time to spend on this baguette",
+        "__paint_color" : "The color to use to paint MetaGraph matches in the visual file",
+        "__suppressed" : "Indicates if an exception should suppress the index file output",
+        "__background_color" : "The color of the background for the visual graph"
+    }
+
+    __pickle_slots__ = {
+        "perf",
+        "filter_names",
+        "pattern_names",
+        "skip_data_comparison",
+        "skip_diff_comparison",
+        "exception",
+        "baked",
+        "toasted",
+        "verbosity",
+        "maxtime",
+        "paint_color",
+        "suppressed",
+        "background_color"
     }
 
     names = {
@@ -57,16 +73,12 @@ class BaguetteRack:
         "extracted" : "Extracted MetaGraphs Python File",
     }
 
-    def __init__(self, working_directory : str | Path | None = None) -> None:
+    def __init__(self, working_directory : str | Path) -> None:
         from pathlib import Path
         from traceback import TracebackException
         from typing import Literal
-        self.__working_directory : Path | None = None
-        self.__index : Path | None = None
-        self.__report : Path | None = None
-        self.__baguette : Path | None = None
-        self.__visual : Path | None = None
-        self.__extracted : Path | None = None
+        from .bakery.source.colors import Color
+        self.__working_directory : Path = Path(working_directory)
         self.__perf : bool = False
         self.__filters : list[str] = []
         self.__patterns : list[str] = []
@@ -77,22 +89,103 @@ class BaguetteRack:
         self.__toasted : bool = False
         self.__verbosity : Literal[0, 1, 2, 3] = 0
         self.__maxtime : float = float("inf")
-        if working_directory is not None and not isinstance(working_directory, str | Path):
+        self.__paint_color : list[Color] | None = None
+        self.__suppressed : bool = False
+        self.__background_color : Color = Color.black
+        if not isinstance(working_directory, str | Path):
             raise TypeError("Expected Path, got " + repr(type(working_directory).__name__))
-        if working_directory is not None:
-            self.set_defaults(Path(working_directory))
+    
+    def __getstate__(self):
+        return {name : getattr(self, name) for name in self.__pickle_slots__}
+    
+    def __setstate__(self, state : dict[str, Any]):
+        self.__filters = []
+        self.__patterns = []
+        for name, value in state.items():
+            setattr(self, name, value)
+
+    @property
+    def working_directory(self) -> Path:
+        """
+        The path where to find this baguette.
+        """
+        return self.__working_directory
+    
+    @property
+    def index(self) -> Path:
+        """
+        The path to the index file which stores the paths to all parts of the baguette.
+        """
+        return self.working_directory / "index.pyt"
+    
+    @property
+    def baguette(self) -> Path:
+        """
+        The path to the baguette file itself (the pickle of the baguette graph) (.pyt).
+        """
+        return self.working_directory / "baguette.pyt"
+
+    @property
+    def visual(self) -> Path:
+        """
+        The path to the visual representation file of the baguette (.gexf).
+        """
+        return self.working_directory / "visual.gexf"
+    
+    @property
+    def extracted(self) -> Path:
+        """
+        The path to the metagraph search results file for this baguette (.pyt).
+        """
+        return self.working_directory / "extracted.pyt"
         
-    def set_defaults(self, working_directory : Path):
+    @property
+    def suppressed(self) -> bool:
         """
-        Sets the default values for parameters given the new working directory.
+        Indicates if a raised exception should suppress the output file writing.
         """
-        working_directory = working_directory.expanduser().resolve()
-        self.__working_directory = working_directory
-        self.__index = working_directory / "index.pyt"
-        self.__baguette = working_directory / "baguette.pyt"
-        self.__visual = working_directory / "visual.gexf"
-        self.__extracted = working_directory / "extracted.pyt"
-        self.check()
+        return self.__suppressed
+
+    @suppressed.setter
+    def suppressed(self, value : bool):
+        if not isinstance(value, bool):
+            raise TypeError(f"Expected bool, got '{type(value).__name__}'")
+        self.__suppressed = value
+
+    @property
+    def paint_color(self) -> list[Color] | None:
+        """
+        The Colors in which to paint the MetaGraph matches found during toasting.
+        The index of the color correspond to the index of the MetaGraph in the "patterns" attribute.
+        None indicates no painting.
+        """
+        return self.__paint_color
+    
+    @paint_color.setter
+    def paint_color(self, value : list[Color] | None):
+        from .bakery.source.colors import Color
+        if value is not None and not isinstance(value, list):
+            raise TypeError("Expected None or list of Colors, got " + repr(type(value).__name__))
+        if isinstance(value, list):
+            for c in value:
+                if not isinstance(c, Color):
+                    raise TypeError("Expected list of Colors, got a " + repr(type(c).__name__))
+        self.__paint_color = value
+
+    @property
+    def background_color(self) -> Color:
+        """
+        The background color that will be used for the visual file.
+        This is used to change the color settings which are too close from the background color in order to make the visual sharper.
+        """
+        return self.__background_color
+    
+    @background_color.setter
+    def background_color(self, value : Color):
+        from .bakery.source.colors import Color
+        if not isinstance(value, Color):
+            raise TypeError(f"Expected Color, got '{type(value).__name__}'")
+        self.__background_color = value
 
     @property
     def maxtime(self) -> float:
@@ -317,228 +410,19 @@ class BaguetteRack:
         if not isinstance(value, bool):
             raise TypeError("Expected bool, got " + repr(type(value).__name__))
         self.__skip_diff_comparison = value
-    
-    def check(self, index : Path | None = None, working_directory : Path | None = None, report : Path | None = None, baguette : Path | None = None, visual : Path | None = None, extracted : Path | None = None):
-        """
-        Checks if the current baguette is well configured.
-        Raises the appropriate exception if it is not.
-        """
-        working_directory = self.working_directory if working_directory is None else working_directory
-
-        index = self.index if index is None else index
-        report = self.report if report is None else report
-        baguette = self.baguette if baguette is None else baguette
-        visual = self.visual if visual is None else visual
-        extracted = self.extracted if extracted is None else extracted
-
-        index = index if index.is_absolute() else working_directory / index
-        report = report if report.is_absolute() else working_directory / report
-        baguette = baguette if baguette.is_absolute() else working_directory / baguette
-        visual = visual if visual.is_absolute() else working_directory / visual
-        extracted = extracted if extracted.is_absolute() else working_directory / extracted
-
-        if working_directory.exists() and not working_directory.is_dir():
-            raise FileExistsError("The given working directory already exists and is not a directory.")
-        if index.exists() and not index.is_file():
-            raise FileExistsError("The given index file path exists and is not a file.")
-        if index.parent != working_directory:
-            raise ValueError("The index file must be located directly in the working directory.")
-    
-    @property
-    def working_directory(self) -> Path:
-        """
-        The path that contains the index file and all other baguette files by default.
-        """
-        if not self.__working_directory:
-            raise RuntimeError("Working directory path has not been set yet.")
-        return self.__working_directory
-
-    @working_directory.setter
-    def working_directory(self, value : str | Path):
-        from pathlib import Path
-        if not isinstance(value, str | Path):
-            raise TypeError("Expected Path, got " + repr(type(value).__name__))
-        if isinstance(value, str):
-            try:
-                value = Path(value)
-            except BaseException as e:
-                raise e from None
-        value = value.expanduser().resolve().absolute()
-        old = self.__working_directory
-        index, report, baguette, visual, extracted = None, None, None, None, None
-        if old:
-            if self.__index and self.__index.is_relative_to(old):
-                index = value / self.__index.relative_to(old)
-            if self.__report and self.__report.is_relative_to(old):
-                report = value / self.__report.relative_to(old)
-            if self.__baguette and self.__baguette.is_relative_to(old):
-                baguette = value / self.__baguette.relative_to(old)
-            if self.__visual and self.__visual.is_relative_to(old):
-                visual = value / self.__visual.relative_to(old)
-            if self.__extracted and self.__extracted.is_relative_to(old):
-                extracted = value / self.__extracted.relative_to(old)
-        self.check(working_directory = value, index = index, report = report, baguette = baguette, visual = visual, extracted = extracted)
-        if old and old.is_dir():
-            from os import renames
-            renames(old, value)
-        self.__working_directory = value
-        if index:
-            self.index = index
-        if report:
-            self.report = report
-        if baguette:
-            self.baguette = baguette
-        if visual:
-            self.visual = visual
-        if extracted:
-            self.extracted = extracted
-
-    @property
-    def index(self) -> Path:
-        """
-        The path to the index file which stores the paths to all parts of the baguette.
-        """
-        if not self.__index:
-            return self.working_directory / "index.json"
-        return self.__index
-    
-    @index.setter
-    def index(self, value : str | Path):
-        from pathlib import Path
-        if not isinstance(value, str | Path):
-            raise TypeError("Expected Path, got " + repr(type(value).__name__))
-        if isinstance(value, str):
-            try:
-                value = Path(value)
-            except BaseException as e:
-                raise e from None
-        value = value.expanduser().resolve().absolute()
-        if not self.__working_directory:
-            self.working_directory = value.parent
-        self.check(index = value)
-        if self.__index and self.__index.is_file():
-            from os import renames
-            renames(self.__index, value)
-        self.__index = value
-    
-    @property
-    def report(self) -> Path:
-        """
-        The path to the Cuckoo report of this baguette (.json).
-        """
-        if self.__report is None:
-            self.__report = self.working_directory / "report.json"
-        return self.__report
-
-    @report.setter
-    def report(self, value : str | Path):
-        from pathlib import Path
-        if not isinstance(value, str | Path):
-            raise TypeError("Expected Path, got " + repr(type(value).__name__))
-        if isinstance(value, str):
-            try:
-                value = Path(value)
-            except BaseException as e:
-                raise e from None
-        value = value.expanduser().resolve().absolute()
-        self.check(report = value)
-        if self.__report and self.__report.is_file():
-            from os import renames
-            renames(self.__report, value)
-        self.__report = value
-
-    @property
-    def baguette(self) -> Path:
-        """
-        The path to the baguette file itself (the pickle of the baguette graph) (.pyt).
-        """
-        if self.__baguette is None:
-            self.__baguette = self.working_directory / "baguette.pyt"
-        return self.__baguette
-
-    @baguette.setter
-    def baguette(self, value : str | Path):
-        from pathlib import Path
-        if not isinstance(value, str | Path):
-            raise TypeError("Expected Path, got " + repr(type(value).__name__))
-        if isinstance(value, str):
-            try:
-                value = Path(value)
-            except BaseException as e:
-                raise e from None
-        value = value.expanduser().resolve().absolute()
-        self.check(baguette = value)
-        if self.__baguette and self.__baguette.is_file():
-            from os import renames
-            renames(self.__baguette, value)
-        self.__baguette = value
-
-    @property
-    def visual(self) -> Path:
-        """
-        The path to the visual representation file of the baguette (.gexf).
-        """
-        if self.__visual is None:
-            self.__visual = self.working_directory / "visual.gexf"
-        return self.__visual
-
-    @visual.setter
-    def visual(self, value : str | Path):
-        from pathlib import Path
-        if not isinstance(value, str | Path):
-            raise TypeError("Expected Path, got " + repr(type(value).__name__))
-        if isinstance(value, str):
-            try:
-                value = Path(value)
-            except BaseException as e:
-                raise e from None
-        value = value.expanduser().resolve().absolute()
-        self.check(visual = value)
-        if self.__visual and self.__visual.is_file():
-            from os import renames
-            renames(self.__visual, value)
-        self.__visual = value
-    
-    @property
-    def extracted(self) -> Path:
-        """
-        The path to the metagraph search results file for this baguette (.pyt).
-        """
-        if self.__extracted is None:
-            self.__extracted = self.working_directory / "extracted.pyt"
-        return self.__extracted
-
-    @extracted.setter
-    def extracted(self, value : str | Path):
-        from pathlib import Path
-        if not isinstance(value, str | Path):
-            raise TypeError("Expected Path, got " + repr(type(value).__name__))
-        if isinstance(value, str):
-            try:
-                value = Path(value)
-            except BaseException as e:
-                raise e from None
-        value = value.expanduser().resolve().absolute()
-        self.check(extracted = value)
-        if self.__extracted and self.__extracted.is_file():
-            from os import renames
-            renames(self.__extracted, value)
-        self.__extracted = value
-    
+        
     def export(self):
         """
         Writes the information of this baguette to the index file.
         Does nothing if the path to the index file has not been set yet.
         """
         from pickle import dump
-        if not self.__working_directory:
-            return
-        self.index.parent.mkdir(parents = True, exist_ok = True)
-        with open(self.index, "wb") as f:
+        self.__working_directory.mkdir(parents = True, exist_ok = True)
+        with open(self.__working_directory / "index.pyt", "wb") as f:
             dump(self, f)
     
     @classmethod
-    def import_from(cls : type[T], path : str | Path) -> T:
+    def import_from(cls : type["BaguetteRack"], path : str | Path) -> "BaguetteRack":
         """
         Loads the information on a baguette from the given index path.
         """
@@ -553,8 +437,25 @@ class BaguetteRack:
                 raise e from None
         path = path.expanduser().resolve()
         with path.open("rb") as f:
-            return load(f)
+            rack : BaguetteRack = load(f)
+        rack.__working_directory = path.parent
+        return rack
     
+    def clean(self):
+        """
+        Cleans the whole baguette. This will remove all of its files and the .bag folder (working directory) if they exist.
+        """
+        from os import remove, rmdir
+        if self.index.is_file():
+            remove(self.index)
+        if self.baguette.is_file():
+            remove(self.baguette)
+        if self.visual.is_file():
+            remove(self.visual)
+        if self.extracted.is_file():
+            remove(self.extracted)
+        if not list(self.working_directory.iterdir()):
+            rmdir(self.working_directory)
 
 
 
